using System;
using System.Data;
using System.Data.SqlClient;

class DataAdapterTest
{
	static void Main()
	{
		string strCon = "Data Source=(local);" + 
			"Integrated Security=SSPI;Database=Northwind";
		SqlConnection cnNorthwind = new SqlConnection(strCon);

		// Create and populate with data the table for the example
		CreateTableAlabala(cnNorthwind);
		PopulateTableAlabala(cnNorthwind);

		string strSelect = "SELECT Id, Name FROM Alabala";

		// Create an adapter and generate its commands
		// with a CommandBuilder
		SqlDataAdapter daAlabala = new SqlDataAdapter(
			strSelect, cnNorthwind);
		SqlCommandBuilder cbAlabala = new SqlCommandBuilder();
		cbAlabala.DataAdapter = daAlabala;

		// Add an event handler for the RowUpdated event
		daAlabala.RowUpdated += new SqlRowUpdatedEventHandler(
			OnRowUpdated);

		// Create and fill a DataSet
		DataSet dsNorthwind = new DataSet();
		daAlabala.Fill(dsNorthwind, "Alabala");

		Console.WriteLine("Initial table contents from the DB:");
		PrintTableAlabala(dsNorthwind);

		// Add some data to the DataSet
		MakeChanges(dsNorthwind);

		Console.WriteLine("Table after adding some records:");
		PrintTableAlabala(dsNorthwind);
			
		Console.WriteLine("Trying to update the table...");
		daAlabala.Update(dsNorthwind, "Alabala");
			
		// Check if there were conflicts and if so, resolve them
		// and call Update(...) again. This time there should be
		// no conflicts, because the conflicting rows are deleted
		// from the DataSet
		if (dsNorthwind.HasErrors)
		{
			Console.WriteLine("There were conflicts!");
			ResolveErrorsInTableAlabala(dsNorthwind);

			Console.WriteLine("Trying to update the table again...");
			daAlabala.Update(dsNorthwind, "Alabala");
			if (dsNorthwind.HasErrors)
				Console.WriteLine("Errors during update!");
			else
				Console.WriteLine("No errors during update.");
		}
			
		// Fill the DataSet with up-to-date data
		daAlabala.Fill(dsNorthwind);
		Console.WriteLine("The table contents from the database:");
		PrintTableAlabala(dsNorthwind);

		// Drop the sample table
		DropTable(cnNorthwind);
	}

	// Prints the data from the DataSet on the console
	static void PrintTableAlabala(DataSet aDataSet)
	{
		Console.WriteLine("ID Name");
		foreach(DataRow row in aDataSet.Tables["Alabala"].Rows)
			Console.WriteLine("{0} {1}", row["Id"], row["Name"]);
		Console.WriteLine();
	}

	// RowUpdate event handler
	private static void OnRowUpdated(object aSender, 
		SqlRowUpdatedEventArgs aEventArgs)
	{
		// Check the Status property of the SqlRowEventArgs
		// argument for indication for errors
		if (aEventArgs.Status == UpdateStatus.ErrorsOccurred)
		{
			// Conflict found. Set error for the updated 
			string errorMessage = string.Format("\nRow \"{0} {1}\"" +
				" conflicts with some row in the database:\n{2}\n",
				aEventArgs.Row["Id"], aEventArgs.Row["Name"],
				aEventArgs.Errors.Message);
			aEventArgs.Row.SetColumnError("Id", errorMessage);

			// Set the Status property to Continue so that the
			// Update(...) method continues with the other rows
			aEventArgs.Status = UpdateStatus.Continue;
		}
	}

	// Add some rows to the DataSet
	static void MakeChanges(DataSet aDataSet)
	{	
		// This row will cause a conflict - it has value
		// 1 for the Id field, but such value is already 
		// present in the table, and Id is a primary key
		DataRow row = aDataSet.Tables["Alabala"].NewRow();
		row["Id"] = 1;
		row["Name"] = "Sasho";
		aDataSet.Tables["Alabala"].Rows.Add(row);
		
		// Add some non-conflict rows
		row = aDataSet.Tables["Alabala"].NewRow();
		row["Id"] = 4;
		row["Name"] = "Tosho";
		aDataSet.Tables["Alabala"].Rows.Add(row);
	}

	// Resolves the conflicts for the rows that have errors
	static void ResolveErrorsInTableAlabala(DataSet aDataSet)
	{
		DataTable alabalaTable = aDataSet.Tables["Alabala"];
		DataRow[] conflictRows = alabalaTable.GetErrors();
		foreach (DataRow row in conflictRows)
		{
			DataColumn[] conflictColumns = row.GetColumnsInError();
			foreach (DataColumn column in conflictColumns)
			{
				string errorMessage = row.GetColumnError(column);
				Console.WriteLine(errorMessage);
			}
			
			// Resolve the conflict by deleting the conflicting row
			row.Delete();
			row.ClearErrors();
		}
	}

	// Creates the table, used in the example
	static void CreateTableAlabala(SqlConnection aConnection)
	{
		string strCreate = "CREATE TABLE Alabala" + 
			"(Id int primary key, Name nvarchar(15))";
		SqlCommand cmdCreate = new SqlCommand(strCreate,
			aConnection);
		try
		{
			aConnection.Open();
			cmdCreate.ExecuteNonQuery();
		}
		catch(Exception ex)
		{
			Console.WriteLine(ex.Message);
		}
		finally
		{
			aConnection.Close();
		}
	}

	// Populates the created table with some sample data
	static void PopulateTableAlabala(SqlConnection aConnection)
	{
		string strInsert = "INSERT INTO Alabala VALUES(1, 'Ivan')";
		SqlCommand cmdInsert = new SqlCommand(strInsert,
			aConnection);
		try
		{
			aConnection.Open();
			cmdInsert.ExecuteNonQuery();

			cmdInsert.CommandText = "INSERT INTO Alabala" + 
				" VALUES(2, 'Pesho')";
			cmdInsert.ExecuteNonQuery();

			cmdInsert.CommandText = "INSERT INTO Alabala" + 
				" VALUES(3, 'Gosho')";
			cmdInsert.ExecuteNonQuery();
		}
		catch(Exception ex)
		{
			Console.WriteLine(ex.Message);
		}
		finally
		{
			aConnection.Close();
		}
	}
	
	// Drop the table at the end of the example
	static void DropTable(SqlConnection aDataSet)
	{
		string strDrop = "DROP TABLE Alabala";
		SqlCommand cmdDrop = new SqlCommand(strDrop, aDataSet);
		try
		{
			aDataSet.Open();
			cmdDrop.ExecuteNonQuery();
		}
		catch(Exception ex)
		{
			Console.WriteLine(ex.Message);
		}
		finally
		{
			aDataSet.Close();
		}
	}
}
